#/* Copyright 2008-2014 Research Foundation State University of New York   */
#
#/* This file is part of QUB Express.                                      */
#
#/* QUB Express is free software; you can redistribute it and/or modify    */
#/* it under the terms of the GNU General Public License as published by   */
#/* the Free Software Foundation, either version 3 of the License, or      */
#/* (at your option) any later version.                                    */
#
#/* QUB Express is distributed in the hope that it will be useful,         */
#/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
#/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
#/* GNU General Public License for more details.                           */
#
#/* You should have received a copy of the GNU General Public License,     */
#/* named LICENSE.txt, in the QUB Express program directory.  If not, see  */
#/* <http://www.gnu.org/licenses/>.                                        */

import numpy
import scipy
import traceback

import ctypes
from ctypes import c_char, c_short, c_int, c_long, c_size_t, c_float, c_double, c_char_p, c_void_p, Structure, POINTER, byref, CFUNCTYPE, sizeof, py_object, cast
c_short_p = POINTER(c_short)
c_int_p = POINTER(c_int)
c_long_p = POINTER(c_long)
c_size_t_p = POINTER(c_size_t)
c_float_p = POINTER(c_float)
c_double_p = POINTER(c_double)
def cdata(arr, typ):
    if arr is None: return arr
    return arr.ctypes.data_as(typ)

try:
    importer = ctypes.windll
    base, ext = '','.dll'
except:
    base, ext = 'lib','.so'
importer = ctypes.cdll

try:
    qubxidlmdl = importer.LoadLibrary(base+'qubxidlmdl'+ext)
except OSError:
    qubxidlmdl = importer.LoadLibrary('@executable_path/../Frameworks/libqubxidlmdl.so') # , mode=ctypes.RTLD_GLOBAL) to be visible from subsequently-loaded libs

qubxidlmdl.qubx_idealize_mdl_context_create.argtypes = ()
qubxidlmdl.qubx_idealize_mdl_context_create.restype = c_void_p
qubxidlmdl.qubx_idealize_mdl_context_free.argtypes = (c_void_p,)
qubxidlmdl.qubx_idealize_mdl_context_free.restype = None
qubxidlmdl.qubx_idealizer_mdl_create.argtypes = (c_void_p,)
qubxidlmdl.qubx_idealizer_mdl_create.restype = c_void_p
qubxidlmdl.qubx_idealizer_mdl_free.argtypes = (c_void_p,)
qubxidlmdl.qubx_idealizer_mdl_free.restype = None
qubxidlmdl.qubx_idealizer_mdl_idealize.argtypes = (c_void_p, c_float_p, c_int, c_int, c_int, c_int, c_float)
qubxidlmdl.qubx_idealizer_mdl_idealize.restype = c_int
qubxidlmdl.qubx_idealizer_mdl_interrupt.argtypes = (c_void_p,)
qubxidlmdl.qubx_idealizer_mdl_interrupt.restype = None
qubxidlmdl.qubx_idealizer_mdl_get_dwells.argtypes = (c_void_p, c_int_p, c_float_p, c_float_p, c_int)
qubxidlmdl.qubx_idealizer_mdl_get_dwells.restype = c_int
qubxidlmdl.qubx_idealizer_mdl_get_dwells_fl.argtypes = (c_void_p, c_int, c_int_p, c_int_p, c_float_p, c_float_p, c_int)
qubxidlmdl.qubx_idealizer_mdl_get_dwells_fl.restype = c_int

class MDLIdealizeContext(object):
    def __init__(self):
        self.obj = qubxidlmdl.qubx_idealize_mdl_context_create()
    def __del__(self):
        qubxidlmdl.qubx_idealize_mdl_context_free(self.obj)
    def new_idealizer(self):
        return MDLIdealizer(self)

class MDLIdealizer(object):
    def __init__(self, ctx):
        self.ctx = ctx
        self.obj = qubxidlmdl.qubx_idealizer_mdl_create(ctx.obj)
    def __del__(self):
        qubxidlmdl.qubx_idealizer_mdl_free(self.obj)
    def idealize(self, data, first=0, Lmin=3, split3=True, combineClose=False, combineDelta=0.5):
        """Returns (firsts, lasts, amps, stds) as numpy arrays.
        
        @param data: numpy.array, dtype float32
        @param first: sample index of beginning of data in file
        @param Lmin: smallest interval (number of samples), default 3
        @param split3: True to also try 3-way splits (slower) (k=2)
"""
        Ndwell = qubxidlmdl.qubx_idealizer_mdl_idealize(self.obj, cdata(data, c_float_p), len(data), Lmin, split3, combineClose, combineDelta)
        firsts = numpy.zeros(shape=(Ndwell,), dtype="int32")
        lasts = numpy.zeros(shape=(Ndwell,), dtype="int32")
        amps = numpy.zeros(shape=(Ndwell,), dtype="float32")
        stds = numpy.zeros(shape=(Ndwell,), dtype="float32")
        qubxidlmdl.qubx_idealizer_mdl_get_dwells_fl(self.obj, first, cdata(firsts, c_int_p), cdata(lasts, c_int_p), cdata(amps, c_float_p), cdata(stds, c_float_p), Ndwell);
        return firsts, lasts, amps, stds
    def interrupt(self):
        qubxidlmdl.qubx_idealizer_mdl_interrupt(self.obj)


        
